# Copyright (C) 2014 Red Hat
#
# This file is part of relval.
#
# wikitcms is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
# Adam Williamson <awilliam@redhat.com>
#
# relval interacts with Fedora release validation events. This file contains
# functions that help implement the 'report-results' sub-command of relval,
# which is a simple utility to report release validation results.

from wikitcms import event as ev
from wikitcms import page as pg
from wikitcms import wiki as wk
from wikitcms import result as rs

import sys
import re

white = re.compile('\s')

def get_confirm():
    confirm = raw_input("Confirm y/n? ")
    while confirm.lower() not in {'y', 'n'}:
        print("Invalid response.")
        confirm = get_confirm()
    return confirm.lower()

def get_release():
    release = raw_input("Which Fedora release would you like to report a "
                            "result for? ")
    try:
        release = int(release)
    except:
        pass
    while release not in range(21,99):
        print("Invalid release. Please input a valid Fedora release number. ")
        release = get_release()
    return release

def get_milestone():
    print("(A)lpha")
    print("(B)eta")
    print("(F)inal")
    m = raw_input("Which milestone would you like to report a milestone for? ")
    if m.lower() == 'a':
        return 'Alpha'
    elif m.lower() == 'b':
        return 'Beta'
    elif m.lower() == 'f':
        return 'Final'
    else:
        print("Invalid choice. Please choose (A)lpha, (B)eta, or (F)inal. ")
        get_milestone()

def get_compose():
    return raw_input("Which compose would you like to report a result for? ")

def get_testtype():
    t = raw_input("Which test type would you like to report a result for? ")
    try:
        return int(t)
    except:
        return None

def get_section():
    section = raw_input("Which section would you like to report a result in? ")
    try:
        return int(section)
    except:
        return None

def get_test():
    test = raw_input("Which test would you like to report a result for? ")
    try:
        return int(test)
    except:
        return None

def get_env():
    env = raw_input("Which environment would you like to report a result for? ")
    try:
        return int(env)
    except:
        return None

def get_result(username):
    status = raw_input("Would you like to report a (P)ass, (F)ail or (W)arn? ")
    while status.lower() not in {'p', 'f', 'w'}:
        status = raw_input("Incorrect choice. Please choose (P)ass, (F)ail or (W)arn. ")
    if status.lower() == 'p':
        status = 'pass'
    if status.lower() == 'f':
        status = 'fail'
    if status.lower() == 'w':
        status = 'warn'
    bugs = raw_input("Enter the IDs of any bugs associated with this report, separated by commas. ")
    if bugs == '':
        bugs = None
    else:
        bugs = white.sub('', bugs)
        bugs = bugs.split(',')
        if bugs == '':
            bugs = None
    return rs.result_text(status, user=username, bugs=bugs)

def get_cont():
    cont = raw_input("Report another result for this (s)ection, or (p)age, "
                     "or this (c)ompose, or (q)uit? ")
    while cont.lower() not in {'s', 'p', 'c', 'q'}:
        print("Invalid choice. Please choose again.")
        cont = get_cont()
    return cont

def report_results(wiki, username, release=None, milestone=None, compose=None,
                  testtype=None, section=None, test=None, auto=True):
    if not release or not milestone or not compose:
        if auto is True:
            curr = wiki.expandtemplates(text='{{CurrentFedoraCompose}}')
            curr = curr.split()
            release = int(curr[0])
            milestone = curr[1]
            compose = curr[2]
            print("Will report results for current compose: " +
                   ' '.join((str(release), milestone, compose)))
            confirm = get_confirm()
            if confirm == 'y':
                pass
            else:
                report_results(wiki, username, auto=False)
        else:
            pass
    if release not in range(21,99):
        release = get_release()

    if milestone not in {'Alpha', 'Beta', 'Final'}:
        milestone = get_milestone()
    while not milestone:
        print("Please choose a valid milestone.")
        milestone = get_milestone()

    while not compose:
        compose = get_compose()

    try:
        testtypes
    except:
        event = ev.ComposeEvent(release, milestone, compose, wiki)
        testtypes = event.testtypes

    if not testtype:
        for i, testtype in enumerate(testtypes):
            print(str(i) + ")    " + testtype)
        t = get_testtype()
        while t not in range(0, len(testtypes)):
            print("Incorrect choice. Please choose one of the test types.")
            t = get_testtype()
        testtype = testtypes[t]

    try:
        page
    except:
        page = pg.ComposePage(wiki, release, milestone, compose, testtype)
    if not section:
        try:
            sections
        except:
            sections = page.results_sections
        if len(sections) == 1:
            section = sections[0]
        else:
            for i, section in enumerate(sections):
                print(str(i) + ")    " + section['line'])
            s = get_section()
            while s not in range(0, len(sections)):
                print("Incorrect choice. Please choose one of the sections.")
                s = get_section()
            section = sections[s]

    try:
        tests
    except:
        tests = page.get_resultrows()
    tests = [t for t in tests if t[1] == section['index']]
    if len(tests) == 0:
        print("No tests in this section. Please choose another.")
        report_results(wiki, username, release, milestone, compose, testtype)
    if len(tests) == 1:
        t = 0
    else:
        for i, test in enumerate(tests):
            if test[2].testcase != test[2].name:
                print(str(i) + ")    " + test[2].testcase + " " + test[2].name)
            else:
                print(str(i) + ")    " + test[2].testcase)
        t = get_test()
        while t not in range(0, len(tests)):
            print("Incorrect choice. Please choose one of the tests.")
            t = get_test()
    test = tests[t][2]
    secid = tests[t][1]

    envs = test.envs
    if len(envs) == 1:
        env = envs[0]
    elif len(envs) > 1:
        for i, env in enumerate(envs):
            print(str(i) + ")    " + env)
        e = get_env()
        while e not in range(0, len(envs)):
            print("Incorrect choice. Please choose an environment.")
            e = get_env()
        env = envs[e]

    results = test.results[env]
    results = [r for r in results if r.user == username]
    if results:
        print("You have already reported result(s) for this test! Do you "
               "still want to report this result?")
        for i, result in results:
            print(str(i) + ")    " + result.status)
        confirm = get_confirm()
        if confirm == 'y':
            pass
        else:
            report_results(wiki, username, release, milestone, compose)
    res = get_result(username)
    print("Will report result for test {} in section {} of {}".format(
            test.testcase, section['line'], page.wikiname))
    try:
        page.add_result(res, test, env, secid)
    except:
        print("Failed!")
    print("Succeeded!")

    cont = get_cont()
    if cont == 'q':
        sys.exit()
    else:
        test = None
    if cont == 'p' or cont == 'c':
        section = None
    if cont == 'c':
        testtype = None
        page = None
        sections = None
        tests = None
    report_results(wiki, username, release, milestone, compose, testtype,
                   section)
